/*  READONLY.C  by Steve Devore    8/3/92  */

/*
	Copyright (c) 1992 Steven P Devore.  All Rights Reserved.

	General permission to re-distribute all or part of this program is granted,
	provided that this statement, including the above copyright notice, is not
	removed.
*/

/*  READONLY.C  */

/*   Purpose:   This program provides a shell to use the Paradox Engine on
				read only tables.  Multiple users can access the file,
				without initializing as a network.

 How it works:

This program chains into the dos interrupt, and looks for the file open
command.  When it finds it, it changes the request to open read only,
deny write access.  This will let others view but not change the file.
*/

#include <stdio.h>
#include <stdlib.h>
#include <share.h>
#include <pxengine.h>
#include <dos.h>


#define TABLENAME "TABLE"

#define DOS 0x21
#define DOSOPEN 0x3D
#define OPENREADONLY DOSOPEN*256 + SH_DENYWR

typedef struct {
	unsigned bp, di, si, ds, es, dx, cx, bx, ax, ip, cs, flags;
} intregs;

void interrupt newdos(intregs r);
void interrupt (*olddos)(void);
void hookdos(void);
void unhookdos(void);

int hooked =FALSE;

void Error(int rc);


void main(void)  {
	TABLEHANDLE tblHandle;
	Error(PXSetDefaults(32,1,2,1,PXDEFAULT,PXDEFAULT));

	hookdos();     			// Open files as read only, deny write.
	Error(PXInit());
	// Insert table open functions that you wish to be read only here
	// Example: 	Error(PXTblOpen(TABLENAME ,&tblHandle,0,0));
	unhookdos();			  // Open files normally

	// Insert table open function that you wish to be read write here.

	// Insert the rest of your program here.

	PXExit();
	exit(0);
}

void Error(int rc)  {
	if (rc != PXSUCCESS) {
		if (hooked) unhookdos();
		printf("Error: %s\n",PXErrMsg(rc));
		PXExit();
		exit(0);
	}
}

void interrupt newdos(intregs r)  {
	//	This interrupt routine is chained into the DOS interrupt, looking
	//  for file open requests.  When it finds one it changes the request
	//  to open read only, deny write access.
	unsigned ax, bx;
	if (_AH == DOSOPEN) r.ax=OPENREADONLY;
	ax = r.ax;
	bx = r.bx;
	r.ax = FP_SEG(olddos);  // Sets up olddos on the stack
	r.bx = FP_OFF(olddos);  // so the retf will "return" to the chained
	_AX = ax; _BX = bx;     // Interrupt routine.

/*  If you do not have an inline assembler, comment out the asm lines and
	uncomment the following line:
	__emit__(0x8B, 0xE5, 0x5D, 0x5F, 0x5E, 0x1F, 0x07, 0x5A, 0x59, 0xCB);
*/
	asm mov sp,bp
	asm pop bp
	asm pop di
	asm pop si
	asm pop ds
	asm pop es
	asm pop dx
	asm pop cx   // Doesn't pop ax, bx so retf will return to chained routine.
	asm retf
}

void hookdos(void)  {						// Sets DOS interrupt to newdos
	olddos = getvect(DOS);
	setvect(DOS,newdos);
	hooked=TRUE;
}

void unhookdos(void)  {					// Changes DOS interrupt to original value
	setvect(DOS,olddos);
	hooked=FALSE;
}
